//=============================================================================
//
// BSD 3-Clause License
//
// Copyright © 2013-2015, DAVID 3D Solutions GbR
// All rights reserved.
//
// Redistribution and use in source and binary forms, 
// with or without modification, are permitted provided that 
// the following conditions are met:
//
// - Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer.
// - Redistributions in binary form must reproduce the above copyright notice, 
//   this list of conditions and the following disclaimer in the documentation 
//   and/or other materials provided with the distribution.
// - Neither the name of DAVID 3D Solutions GbR nor the names of its contributors 
//   may be used to endorse or promote products derived from this software 
//   without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//=============================================================================

#pragma once

#ifndef DAVID_SDK_D4LIB_HELPERS_H
#define DAVID_SDK_D4LIB_HELPERS_H

#include "davidSDK/d4lib.h"
//#include "davidSDK/Exceptions.h"
#include <memory>

/// C++ helper functions for d4lib.
namespace d4 { 

//*****************************************************************************
/// @addtogroup LowLevelGroup
/// @{
/// @defgroup d4lib_helpers Helpers for DAVID Low Level Library (d4lib)
/// Provides C++ helper functions for the DAVID Low Level Library (d4lib).
/// @{
//*****************************************************************************

/// Encapsulates a native d4lib pointer into a shared_ptr object.
/// @param[in] ptr Valid pointer of a d4lib object.
/// @return Shared pointer.
template <class T>
//std::shared_ptr<T> MakeShared(T* ptr, bool checkPtr=true)
std::shared_ptr<T> MakeShared(T* ptr)
{
	//if (checkPtr && ptr == nullptr) ThrowException(david::Error_InvalidArgument, "Pointer is nullptr.", "MakeShared");
	return std::shared_ptr<T>(ptr, d4mem_decRefCount);
}


/// C++ functions related to d4img.
namespace img {

/// Create new image object (#d4img_newImage).
/// @return shared_ptr.
inline std::shared_ptr<d4img> newImage()
{
	return MakeShared(d4img_newImage());
}

} // img


/// C++ functions related to d4fimg.
namespace fimg {

/// Create new float image object (#d4fimg_newImage).
/// @return shared_ptr.
inline std::shared_ptr<d4fimg> newImage()
{
	return MakeShared(d4fimg_newImage());
}

} // fimg


/// C++ functions related to d4projector.
namespace projector {

/// Create new extended display projector (#d4projector_newExtendedDisplayProjector).
/// @return shared_ptr.
inline std::shared_ptr<d4projector> newExtendedDisplayProjector()
{
	return MakeShared(d4projector_newExtendedDisplayProjector());
}

} // projector


/// C++ functions related to d4cam.
namespace cam {

/// Create new direct show camera object (#d4cam_newDirectShowCamera).
/// @return shared_ptr.
inline std::shared_ptr<d4cam> newDirectShowCamera()
{
	return MakeShared(d4cam_newDirectShowCamera());
}

} // d4cam


/// C++ functions related to d4calib.
namespace calib {

/// Create new camera calibration object (#d4calib_newCameraCalibration).
/// @return shared_ptr.
inline std::shared_ptr<d4calib> newCameraCalibration()
{
	return MakeShared(d4calib_newCameraCalibration());
}

} // calib


/// C++ functions related to d4calib.
namespace calib {

/// Create new projector calibration object (#d4calib_newProjectorCalibration).
/// @return shared_ptr.
inline std::shared_ptr<d4calib> newProjectorCalibration()
{
	return MakeShared(d4calib_newProjectorCalibration());
}

} // calib


/// C++ functions related to d4sls.
namespace sls {

/// Create new structured light scanner object (#d4sls_newStructuredLightScanner).
/// @return shared_ptr.
inline std::shared_ptr<d4sls> newStructuredLightScanner()
{
	return MakeShared(d4sls_newStructuredLightScanner());
}

} // sls


/// C++ functions related to d4mesh.
namespace mesh {

/// Create new mesh object (#d4mesh_newMesh).
/// @return shared_ptr.
inline std::shared_ptr<d4mesh> newMesh()
{
	return MakeShared(d4mesh_newMesh());
}

} // mesh


/// @} d4lib
/// @} LowLevelGroup

} // d4

#endif // DAVID_SDK_D4LIB_HELPERS_H
